<?xml version="1.0" encoding="UTF-8"?>
<gui xmlns="http://www.kde.org/standards/kxmlgui/1.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" name="kinfocenter" version="4" xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0   http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
  <MenuBar>
    <Menu name="help">
      <Action name="help_about_view" append="about_merge"/>
      <Action name="help_about_module" append="about_merge"/>
    </Menu>
  </MenuBar>
  <ToolBar name="mainToolBar" fullWidth="true" hidden="false" noMerge="1">
    <text context="@title:menu">Main Toolbar</text>
    <Action name="helpModule"/>
    <Action name="helpMenu"/>
  </ToolBar>
</gui>
